#ifndef PropertyValue_h_Included
#define PropertyValue_h_Included

#include <QVariant>
#include <QString>

enum ValueSource
{
   Source_Undefined,
   Source_Preset,
   Source_Default,
   Source_User,
   Source_Derived,
};

template<typename T>
struct PropertyValue
{
   PropertyValue() : value(), source(Source_Undefined) {}
   PropertyValue(T val) : value(val), source(Source_Undefined) {}
   PropertyValue(T val, ValueSource src) : value(val), source(src) {}
   PropertyValue(const PropertyValue<T>& rhs) : value(rhs.value), source(
         rhs.source) {}

   operator T() const { return value; }

   bool operator==(const PropertyValue<T>& rhs) const { return rhs.value == value &&
      rhs.source == source; }

   /** Undefine value.
    *
    * source is set to Source_Undefined.
    * */
   void undefine() { source = Source_Undefined; }

   /** Is the value defined?
    *
    * Meaning source is not Source_Undefined.
    */
   bool isDefined() const { return source != Source_Undefined; }

   /** Define getters, due to SIP shortcoming. */
   T getValue() const { return value; }
   ValueSource getSource() const { return source; }

   T value;
   ValueSource source;
};

typedef PropertyValue<QString> StringValue;

Q_DECLARE_METATYPE(StringValue);

#endif
